// @ myth - client - window.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "client/window.h"
#include "common/error.h"
#include "common/format.h"

// @@  data

static SCREEN* screen = NULL;
static coor RemainCorner = coor(0, 0); // g++ do not like Origo...
static coor RemainVolume = coor(0, 0);

// @@ function

void SelectScreen(SCREEN* screen)
{
    ::screen = screen;
    ::RemainCorner = Origo;
    ::RemainVolume = screen->size();
}

// @@ class

// @@@ * class WINDOW

// @@@@ * void WINDOW::print(coor pos, const char* string)
//
// Print a string in the window, and clear to end of window.

void WINDOW::print(coor pos, const char* string)
{
    while(*string != 0)
    {
	plot(pos, *string);
	pos.x++;
	string++;
    }
    screen->ClearLine(ToScreen(pos), ToScreen(coor(size().x, pos.y)));
}

// @@@@ * void WINDOW::print(coor pos, STRING string)

void WINDOW::print(coor pos, STRING  string)
{ print(pos, string()); }

// @@@@ WINDOW::WINDOW(edge where,
//
// Create a new window.
//
// The new window will cover some part of the screen, not covered by
// other windows. The position of the new window on the unused area on
// the screen, will depend on the value of the first argument
//    1) Position of the window
//    WINDOW_LEFT, WINDOW_RIGHT, WINDOW_BOTTOM, WINDOW_TOP
//        Create window as fare at the specified side as possible.
//    WINDOW_ALL
//        Let new window fill up all the unused part of the screen.
//    WINDOW_NONE
//        Create an empty window.
// The remaining three arguments is used to determinate the size of
// the window.
//    2) Minimum size of window (default 1).
//       If theres less free space than this, no space will be allocated.
//    3) Per cent of free space to use
//       (default 0, and at least as specified in 2).
//    4) Minimal size of free space, before attempting to use any.
//       (default 0).
// 
// Uses the static variables RemainVolume and RemainCorner to keep
// track of the unoccupied part of the screen.

WINDOW::WINDOW(edge where, int size = 1, int percent = 0, int minimum = 0)
{
    screen = ::screen;

    switch(where)
    {
    case WINDOW_LEFT:
    case WINDOW_RIGHT:
	if(RemainVolume.x < minimum)
	{
	    corner = Origo;
	    volume = Origo;
	}
	else
	{
	    volume = coor(max(size, (RemainVolume.x * percent) / 100),
			  RemainVolume.y);
	    RemainVolume.x -= volume.x;
	    if(where == WINDOW_LEFT)
	    {
		corner = RemainCorner;
		RemainCorner.x += volume.x;
	    }
	    else
		corner = coor(RemainCorner.x + RemainVolume.x,
			      RemainCorner.y);
	}	
	return;
    case WINDOW_TOP:
    case WINDOW_BOTTOM:
	if(RemainVolume.y < minimum)
	{
	    corner = Origo;
	    volume = Origo;
	}
	else
	{
	    volume = coor(RemainVolume.x,
			  max(size, (RemainVolume.y * percent) / 100));
	    RemainVolume.y -= volume.y;
	    if(where == WINDOW_TOP)
	    {
		corner = RemainCorner;
		RemainCorner.y += volume.y;
	    }
	    else
		corner = coor(RemainCorner.x,
			      RemainCorner.y + RemainVolume.y);
	}	
	return;
    case WINDOW_ALL:
	corner = RemainCorner;
	volume = RemainVolume;
	::RemainVolume = Origo;
	return;
    case WINDOW_NONE:
	corner = Origo;
	volume = screen->size();
	return;
    default:
	corner = Origo;
	volume = Origo;
	bug.error("WINDOW::WINDOW()", format("Unknown edge %d", where));
    }
    return;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
