// @ myth - common - error.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk
//
// Error handling.
//
// Initialise error handling with:
//    bug.open(name);	     // name is the name of the logfile to be used.
//
// Now you can report warnings, errors and panic with:
//    bug.warning("where", "what"); // Not a real bug.
//    bug.error("where", "what");   // A real bug, but it might be recovered.
//    bug.panic("where", "what");   // An unrecoverable bug.
// 
// Where "where" is the name of the rutine, and "what" is a
// description of what went wrong.
//
// `bug' is a global, defined in this file.

// @@ include

#include "common/error.h"
#include "common/format.h"
#include <time.h>

extern pfVV set_new_handler(pfVV handler);

// @@ forward

static void new_handler();

// @@ class BUG

// @@@ BUG::open(STRING name)
//
// Initialise error handler.

void BUG::open(STRING name)
{
    BUFFER::open(name);

    if(!GoodOutput())
    {
	perror("BUG::open");
	(void) fprintf(stderr,
		       "Can't open ErrorLog. Contact the Myth Master.\n");
	exit(1);
    }
    warning("BUG::open()", "Initialised");
    set_new_handler(new_handler);
}

// @@@ void BUG::warning(const char* where, const char* what)
//
// Unusual event - write a warning on errfile

void BUG::warning(const char* where, const char* what)
{
    const char* string;    

    const int MaxError = 999;
    static int count = 0;

    if(count > MaxError)
    {
	static const char* const final = "To many errors";
	string = final;
    }
    else
    {
	long now;
	(void) time(&now);
	string = format("%s%s - %s. (%s)\n\n", ctime(&now), where, what, 
			(errno < sys_nerr ? sys_errlist[errno] : "Who Knows?")
			).string();
    }
    if(!GoodOutput())
	perror(format("%s - %s", where, what).string());
    else
    {
	put(string, strlen(string));
	flush();
	(void) fsync(fileno());
    }
    if(count > MaxError)
    {
	(void) fprintf(stderr,
		       "To many internal errors. Contact the Myth Master.\n");
	exit(1);
    }
    else
	count++;

    errno = 0;
}

// @@@ void BUG::warning(const char* where, STRING what)

void BUG::warning(const char* where, STRING what)
{ warning(where, what()); }

// @@@ void BUG::warning(STRING where, const char* what)

void BUG::warning(STRING where, const char* what)
{ warning(where(), what); }

// @@@ void BUG::error(const char* where, const char* what)
//
// Some program bug is found. Report to errfile and continue.

void BUG::error(const char* where, const char* what)
{ warning(format("error:%s:", where), what); }

// @@@ void BUG::error(const char* where, STRING what)

void BUG::error(const char* where, STRING what)
{ error(where, what()); }

// @@@ void BUG::fatal(const char* where, const char* what)
//
// Some program bug is found. Report to errfile and abort.

void BUG::fatal(const char* where, const char* what)
{
    warning(format("fatal:%s", where), what);
    (void) fprintf(stderr, "Yarg!: %s... Contact the Myth Master.\n", what);
    exit(1);
}

// @@@ void BUG::fatal(const char* where, STRING what)

void BUG::fatal(const char* where, STRING what)
{ fatal(where, what()); }

// @@@ void BUG::panic(const char* where, const char* what)
//
// Some program bug is found. Report to errfile and abort.

void BUG::panic(const char* where, const char* what)
{
    warning(format("panic:%s", where), what);
    (void) fprintf(stderr, "panic! Contact the Myth Master.\n");
    // abort(); 
    exit(1);
}

// @@@ BUG::BUG()    
    
BUG::BUG() : (BUFFER_WRITE)
{ }

// @@@ BUG::~BUG()

BUG::~BUG()
{ }
 
// @@ BUG bug

BUG bug;

// @@ new_handler
//
// Tell what went wrong in "new".

static void new_handler()
{ bug.panic("new", "memory exhausted"); }

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
