// @ myth - common - range.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/range.h"
#include "common/error.h"

// @@ const

// Any const char* will be a infinite range
const char* INFINITE = "";

// @@ type

// @@@ * class range

// @@@@ * bool range::member(long value)

bool range::member(long value)
{
    if(bottomless)
	return topless ? TRUE : (value <= top);
    if(topless)
	return value >= bottom;

    return value >= bottom && value <= top;
}

// @@@@ range::range(long min, long max)

range::range(long min, long max)
{
    bottomless = topless = FALSE;
    bottom = min;
    top = max;
}

// @@@@ range::range(const char* min, long max)

range::range(const char* min, long max)
{
    (void) min;
    bottomless = TRUE;
    topless = FALSE;
    top = max;
}

// @@@@ range::range(long min, const char* max)

range::range(long min, const char* max)
{
    (void) max;
    bottomless = FALSE;
    topless = TRUE;
    bottom = min;
}

// @@@@ range::range(const char* min, const char* max)

range::range(const char* min, const char* max)
{
    (void) min, max;
    bottomless = topless = TRUE;
}

// @@@@ range::range(const range& r)

range::range(const range& r)
{
    bottomless = r.bottomless;
    topless = r.topless;
    bottom = r.bottom;
    top = r.top;
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
