// @ myth - server - body.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "server/corpse.h"
#include "server/weapon.h"
#include "server/rnd.h"
#include "server/monster.h"
#include "server/player.h"
#include "common/binary.h"
#include "common/parse.h"
#include "common/error.h"
#include "server/ray.h"
#include "common/format.h"

// @@ Body Weapons

FIST TheFist;
WRAITH_FIST TheWraithFist;
FUNGII TheFungii;
TEETH TheTeeth;
COCK TheCock;
TAIL TheTail;
ACID TheAcid;

// @@ BODY

// @@@ int BODY::health()
//
// Number from 0 to 25 showing players health

int BODY::health()
{
    return (25 * HitPoint) / max(MaxHitPoint, 1L);
}

// @@@ void BODY::kill()

void BODY::kill()
{
    HitPoint = -1;
    when = watch() + rnd(5 * 60, 30 * 60);
    gone = when + rnd(60 * 60, 3 * 24 * 60 * 60);
}

// @@@ cont BODY::hurt(int power, int type, OBJECT* weapon,

cont BODY::hurt(int power, int type, OBJECT* weapon,
		MONSTER* attack, MONSTER* defence, LEVEL*)
{
    power += weapon->DamageBonus(attack, defence);

    power -= (power * armor()) / 100;

    if((type & DAMAGE_FLAME) && CheckAbility(ABILITY_FIRE_PROTECTION))
	type &= ~DAMAGE_FLAME;

    if(type & DAMAGE_COLD)
    {
	if(CheckAbility(ABILITY_COLD_PROTECTION))
	    type &= ~DAMAGE_COLD;
	else if(ColdBlood())
	    power *= 5;
    }
    if((type & DAMAGE_ACID) && CheckAbility(ABILITY_ACID_PROTECTION))
	type &= ~DAMAGE_ACID;

    if((type & DAMAGE_POISON))
    {
	if(!CheckAbility(ABILITY_POISON_PROTECTION) && power > 0)
	{
	    defence->message("You are poisoned.");
	    defence->AddStrength(-rnd(0, MyLog(power + 1)));
	}
	type &= ~DAMAGE_POISON;
    }
    if((type & DAMAGE_STONE))
    {
	if(!CheckAbility(ABILITY_STONE_PROTECTION) &&
	   !CheckAbility(ABILITY_STONED))
	    AddAbility(new COUNTER_STONED(1L));

	type &= ~DAMAGE_STONE;
    }
    if(type != DAMAGE_HARMLESS)
	HitPoint -= power;
    
    return dead() ? DONE : CONTINUE;
}

// @@@ void BODY::SetExperience(int exp)

void  BODY::SetExperience(int exp)
{
    while(experience > exp)
    {
	int hp = constitution() +
	    rnd(experience + 1) + rnd(2 * constitution() + 1);
	HitPoint -= hp;
	MaxHitPoint -= hp;
	experience--;
    }
    while(experience < exp)
    {
	int hp = constitution() +
	    rnd(experience + 1) + rnd(2 * constitution() + 1);
	HitPoint += hp;
	MaxHitPoint += hp;
	experience++;
    }
    HitPoint = max(HitPoint, 1);
    MaxHitPoint = max(MaxHitPoint, 1);
}

// @@@ void BODY::heal(int hp)

void BODY::heal(int hp)
{
    if(HitPoint < MaxHitPoint)
	HitPoint = min(HitPoint + hp, MaxHitPoint);
    else
	MaxHitPoint += rnd(1, rnd(1, hp));
}

// @@@ void BODY::eat(LEVEL* level, MONSTER* monster)

void BODY::eat(LEVEL* level, MONSTER* monster)
{
    if(monster->feed(FoodValue(), CHECK) == DONE)
    {
	monster->message("You choke over your food.");
	level->WatchMonster(monster, "%s chokes");
	monster->kill(format("choked over a %s", name()), level);
    }
    else
    {
	if(tainted())
	{
	    monster->message("Ulch -- that meat was tainted!");
	    monster->AddAbility(new COUNTER_SICK(rnd(5, 50)));
	}
	else if(LivingLook() == monster->look())
	    monster->message("You cannibal! You will be sorry for this!");

	if(poisonous())
	{
	    monster->message("Ecch --- that must have been poisonous!");
	    
	    if(monster->CheckAbility(ABILITY_POISON_PROTECTION))
		monster->message("You don't seem affected by the poison.");
	    else
		monster->AddStrength(-rnd(1, MyLog(FoodValue() / 60 + 2)));
	}
    }
}

// @@@ void BODY::effect(LEVEL*, MONSTER*)

void BODY::effect(LEVEL*, MONSTER*)
{ /* Default: No effect */ }

// @@@ QUESTION* BODY::DoEffect(LEVEL* level, PLAYER* player, CHANNEL*)

QUESTION* BODY::DoEffect(LEVEL* level, PLAYER* player, CHANNEL*)
{
    effect(level, player);
    return NULL;
}

// @@@ void BODY::apply(LEVEL* level, MONSTER* monster)

void BODY::apply(LEVEL* level, MONSTER* monster)
{
    eat(level, monster);
    effect(level, monster);
}

// @@@ QUESTION* BODY::DoApply(LEVEL* level, PLAYER* player, CHANNEL* channel)

QUESTION* BODY::DoApply(LEVEL* level, PLAYER* player, CHANNEL* channel)
{
    eat(level, player);
    return DoEffect(level, player, channel);
}

// @@@ cont BODY::feed(int value, force flag)

cont BODY::feed(int value, force flag)
{
    bool full = (flag == CHECK) && FoodStatus() > 7 && CanChoke();
    food += value;
    return (food < stomach() || !full) ? CONTINUE : DONE;
}

// @@@ int BODY::strength()

int BODY::strength()
{
    int str = MyStrength + GetValue(ABILITY_STRENGTH);

    return min(str - max(3 - FoodStatus(), 0), 25);
}

// @@@ void BODY::AddStrength(int str)

void BODY::AddStrength(int str)
{
    if(str != 0)
    {
	MyStrength += str;
	MaxStrength = max(MaxStrength, MyStrength);
    }
}

// @@@ void BODY::RestoreStrength()

void BODY::RestoreStrength()
{
    if(MyStrength < MaxStrength)
	MyStrength = MaxStrength;
}

// @@@ const char* BODY::BodyName()

const char* BODY::BodyName()
{
    bug.error("BODY::BodyName", "virtual");
    return "bug";
}

// @@@ const char* BODY::name()

const char* BODY::name()
{
    if(dead())
    {
	STRING string = format("dead %s", BodyName());
	const char* text = string.string();
	return text;
    }
    else
	return BodyName();
}

// @@@ const char* BODY::PluralName()

const char* BODY::PluralName()
{
    STRING string = format("%ss", name());
    const char* text = string.string();
    return text;
}

// @@@ const char* BODY::ClassName()

const char* BODY::ClassName()
{ return name(); }

// @@@ const char* BODY::PluralClassName()

const char* BODY::PluralClassName()
{
    STRING string = format("%ss", ClassName());
    const char* text = string.string();
    return text;
}

// @@@ OBJECT* BODY::weapon()

OBJECT* BODY::weapon()
{
    bug.error("BODY::weapon", "virtual");
    return &TheFist;
}

// @@@ int BODY::speed()

int BODY::speed()
{
    int fast = BodySpeed() + GetValue(ABILITY_FAST);
    int slow = GetValue(ABILITY_SLOW) + 1;

    if(slow > 1)
	return fast / slow + 1;
    else
	return fast;
}

// @@@ int BODY::sight()

int BODY::sight()
{
    if(CheckAbility(ABILITY_BLIND))
	return 1;
    else
	return max(view() + GetValue(ABILITY_SIGHT), 1);
}

// @@@ void BODY::hiss(LEVEL* level, MONSTER* monster)

void BODY::hiss(LEVEL* level, MONSTER* monster)
{
    if(monster->CheckAbility(ABILITY_HISS))
	level->hiss(monster);
    else    
    {
	monster->message("You make a fool out of yourself.");
	level->WatchMonster(monster, "%s makes a funny hissing sound.");
    }
}

// @@@ void BODY::breathe(LEVEL* level, MONSTER* monster, coor pos)

void BODY::breathe(LEVEL* level, MONSTER* monster, coor pos)
{
    if(monster->CheckAbility(ABILITY_BREATHE_FIRE))
    {
	if(monster->MayFight())
	{
	    monster->UseFightPoint(150);
	    monster->message("You breathe fire.");
	    level->WatchMonster(monster, "%s breathes fire.");
	    level->zap(monster, new RAY_FLAME(monster->experience()), pos);
	}
	else
	{
	    monster->UseFightPoint(15);
	    monster->message("You breathe smoke.");
	    level->WatchMonster(monster, "%s breathes smoke.");
	}
    }
    else
    {
	monster->UseFightPoint(5);
	monster->message("Your bad breath don't impress anybody.");
	level->WatchMonster(monster, "%s blows in the air.");
    }
}

// @@@ void BODY::status(PLAYER* player)

void BODY::status(PLAYER* player)
{
    player->SendStatus(STATUS_STRENGTH, strength());
    player->SendStatus(STATUS_HEALTH, health());
    player->SendStatus(STATUS_FOOD, FoodStatus());
}

// @@@ void BODY::DigestFood(MONSTER* monster, int val = 1)

void BODY::DigestFood(MONSTER* monster, int val = 1)
{
    int OldStatus = FoodStatus();

    food -= digest() * val;

    if(OldStatus != FoodStatus() && FoodStatus() < 6)
    {
	switch(FoodStatus())
	{
	case 1:
	    monster->message("Food! Now!");
	    break;
	case 2:
	    monster->message("You are getting weak from lack of food.");
	    break;
	case 3:
	    monster->message("You really could use some food now.");
	    break;
	case 4:
	    monster->message("You are getting hungry.");
	    break;
	case 5:
	    monster->message("Lunchtime?");
	    break;
	default:
	    monster->message("You are starving...");
	}
    }
}

// @@@ void BODY::update(LEVEL* level, MONSTER* monster)

void BODY::update(LEVEL* level, MONSTER* monster)
{
    if(food < 0)
    {
	monster->message("You die from hunger.");
	level->WatchMonster(monster, "%s dies from hunger.");
	monster->kill("starved to death", level);
    }
    else if(FoodStatus() < 2)
    {
	if(monster->MayMove() && rnd.chance(5))
	{
	    monster->UseFightPoint(rnd(10000));
	    monster->UseMovePoint(rnd(10000));
	}
    }
    else if(monster->CheckAbility(ABILITY_INFECTED))
    {
	if(--HitPoint < 1)
	{
	    monster->message("You die from an infection.");
	    level->WatchMonster(monster, "%s dies from an infection.");
	    monster->kill("killed by a fungus infection.", level);
	}
    }
    else if(HitPoint < MaxHitPoint && monster->CanMove())
    {
	DigestFood(monster, 2);	// Use extra food to regenerate

	if(monster->CheckAbility(ABILITY_SICK))
	    /* Do Nothing */;
	else if(constitution() + experience > 35)
	    HitPoint += rnd(1, constitution() + experience - 35);
	else if(rnd.chance(37 - constitution() - experience))
	    HitPoint++;
    }

    count.update(level, monster);
}

// @@@ void BODY::save(BINARY* binary)

void BODY::save(BINARY* binary)
{
    // We must save the type first, in order to be able to restore it
    binary->PutShort(type());
    // Store common stuff
    binary->PutLong(when);
    binary->PutShort(HitPoint);
    binary->PutShort(MaxHitPoint);
    binary->PutLong(food);
    binary->PutShort(MyStrength);
    binary->PutShort(MaxStrength);
    binary->PutShort(experience);
    count.save(binary);
}

// @@@ BODY::BODY(BINARY* binary)

BODY::BODY(BINARY* binary)
{
    when = binary->GetLong();
    HitPoint = binary->GetShort();
    MaxHitPoint = binary->GetShort();
    food = binary->GetLong();
    MyStrength = binary->GetShort();
    MaxStrength = binary->GetShort();
    experience = binary->GetShort();
    count.load(binary);
}

// @@@ BODY::BODY(int str)

BODY::BODY(int str)
{
    when = -1L;
    HitPoint = MaxHitPoint = 1;
    food = 2500;
    experience = 0;
    SetStrength(str);
}

// @@ BODY* ::LoadBody(BINARY* binary)

BODY* LoadBody(BINARY* binary)
{
    switch(binary->GetShort())
    {
    case BODY_TYPE_HUMAN:
	return new BODY_HUMAN(binary);
    case BODY_TYPE_ACID_BLOB:
	return new BODY_ACID_BLOB(binary);
    case BODY_TYPE_BAT:
	return new BODY_BAT(binary);
    case BODY_TYPE_RAT:
	return new BODY_RAT(binary);
    case BODY_TYPE_GOBLIN:
	return new BODY_GOBLIN(binary);
    case BODY_TYPE_HOBGOBLIN:
	return new BODY_HOBGOBLIN(binary);
    case BODY_TYPE_ORC:
	return new BODY_ORC(binary);
    case BODY_TYPE_DRAGON:
	return new BODY_DRAGON(binary);
    case BODY_TYPE_LIZARD:
	return new BODY_LIZARD(binary);
    case BODY_TYPE_COCKATRICE:
	return new BODY_COCKATRICE(binary);
    case BODY_TYPE_WRAITH:
	return new BODY_WRAITH(binary);
    case BODY_TYPE_ELF:
	return new BODY_ELF(binary);
    case BODY_TYPE_TROLL:
	return new BODY_TROLL(binary);
    case BODY_TYPE_FUNGUS:
	return new BODY_FUNGUS(binary);
    case BODY_TYPE_END:
    case BODY_TYPE_ERROR:
    default:
	bug.panic("::LoadBody", "Unknown body");
    }
    return NULL;		// Not Reached!
}

// @@ BODY* ::CreateBody(int exp)

BODY* CreateBody(int)
{
    switch(rnd(BODY_TYPE_HUMAN, BODY_TYPE_END - 1))
    {
    case BODY_TYPE_HUMAN:
	return new BODY_HUMAN;
    case BODY_TYPE_ACID_BLOB:
	return new BODY_ACID_BLOB;
    case BODY_TYPE_BAT:
	return new BODY_BAT;
    case BODY_TYPE_RAT:
	return new BODY_RAT;
    case BODY_TYPE_GOBLIN:
	return new BODY_GOBLIN;
    case BODY_TYPE_HOBGOBLIN:
	return new BODY_HOBGOBLIN;
    case BODY_TYPE_ORC:
	return new BODY_ORC;
    case BODY_TYPE_DRAGON:
	return new BODY_DRAGON;
    case BODY_TYPE_LIZARD:
	return new BODY_LIZARD;
    case BODY_TYPE_COCKATRICE:
	return new BODY_COCKATRICE;
    case BODY_TYPE_WRAITH:
	return new BODY_WRAITH;
    case BODY_TYPE_ELF:
	return new BODY_ELF;
    case BODY_TYPE_TROLL:
	return new BODY_TROLL;
    case BODY_TYPE_FUNGUS:
	return new BODY_FUNGUS;
    case BODY_TYPE_END:
    case BODY_TYPE_ERROR:
    default:
	bug.panic("::LoadBody", "Unknown body");
    }
    return NULL;		// Not Reached!
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
