// @ myth - server - corpse.h
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ begin

#ifndef CORPSE_HEADER
#define CORPSE_HEADER

// @@ include

#include "server/body.h"
#include "server/monster.h"
#include "server/rnd.h"

// @@ BODY_HUMAN

class BODY_HUMAN : public BODY
{
protected:
    const char* BodyName()
    { return "human"; }
    char LivingLook()
    { return '@'; }
    int view()
    { return 3; }
    int constitution()
    { return 15; }
    int FoodValue()
    { return 25 * 60; }
    int stomach()
    { return 60 * 60; }
    int armor()
    { return 15; }
public:
    object_type LookType()
    { return MONSTER_PLAYER; }
    int MaxRing()
    { return 2; }
    body_type type()
    { return BODY_TYPE_HUMAN; }
    long size()
    { return 5000; }
    long weight()
    { return 3000; }
    OBJECT* weapon()
    { return &TheFist; }
    int damage()
    { return 15; }
    BODY_HUMAN(BINARY* binary) : (binary) { }
    BODY_HUMAN() : (10) { }
};

// @@ BODY_FUNGUS

class BODY_FUNGUS : public BODY
{
private:
    int TimeLeft;
    void ReCharge()
    { TimeLeft = rnd(50, 450); }
protected:
    int BodySpeed()
    { return 10; }
    const char* BodyName()
    { return "fungus"; }
    char LivingLook()
    { return 'f'; }
    int view()
    { return 1; }
    int constitution()
    { return 15; }
    int FoodValue()
    { return 1 * 60; }
    int stomach()
    { return 1 * 60; }
    int digest()
    { return 0; }
    bool poisonous()
    { return TRUE; }
    int armor()
    { return 65; }
public:
    object_type LookType()
    { return MONSTER_FUNGUS; }
    bool CanMove(MONSTER*)
    { return FALSE; }
    body_type type()
    { return BODY_TYPE_FUNGUS; }
    long size()
    { return 25; }
    long weight()
    { return 25; }
    OBJECT* weapon()
    { return &TheFungii; }
    int damage()
    { return 3; }
    void update(LEVEL* level, MONSTER* monster);
    void save(BINARY* binary);
    BODY_FUNGUS(BINARY* binary);
    BODY_FUNGUS();
};

// @@ BODY_TROLL

class BODY_TROLL : public BODY
{
protected:
    int BodySpeed()
    { return 125; }
    const char* BodyName()
    { return "troll"; }
    char LivingLook()
    { return 'T'; }
    int view()
    { return 3; }
    int constitution()
    { return 16; }
    int FoodValue()
    { return 60 * 60; }
    int stomach()
    { return 150 * 60; }
    int armor()
    { return 25; }
public:
    object_type LookType()
    { return MONSTER_TROLL; }
    int MaxRing()
    { return 2; }
    body_type type()
    { return BODY_TYPE_TROLL; }
    long size()
    { return 7500; }
    long weight()
    { return 8000; }
    OBJECT* weapon()
    { return &TheFist; }
    int damage()
    { return 20; }
    BODY_TROLL(BINARY* binary) : (binary)  { }
    BODY_TROLL() : (15) { }
};

// @@ BODY_ELF

class BODY_ELF : public BODY
{
protected:
    int BodySpeed()
    { return 200; }
    const char* BodyName()
    { return "elf"; }
    char LivingLook()
    { return 'E'; }
    int view()
    { return 5; }
    int constitution()
    { return 12; }
    int FoodValue()
    { return 20 * 60; }
    int stomach()
    { return 40 * 60; }
    int armor()
    { return 5; }
public:
    object_type LookType()
    { return MONSTER_ELF; }
    int MaxRing()
    { return 4; }
    body_type type()
    { return BODY_TYPE_ELF; }
    long size()
    { return 4000; }
    long weight()
    { return 2000; }
    int damage()
    { return 12; }
    OBJECT* weapon()
    { return &TheFist; }
    BODY_ELF(BINARY* binary) : (binary)  { }
    BODY_ELF() : (8) { }
};

// @@ BODY_WRAITH

class BODY_WRAITH : public BODY
{
protected:
    const char* BodyName()
    { return "wraith"; }
    char LivingLook()
    { return 'W'; }
    int view()
    { return 4; }
    int constitution()
    { return 10; }
    int FoodValue()
    { return 20 * 60; }
    int stomach()
    { return 40 * 60; }
    void effect(LEVEL*, MONSTER* monster)
    { monster->GainExperience(rnd(1, experience)); }
    int armor()
    { return 0; }
public:
    object_type LookType()
    { return MONSTER_WRAITH; }
    int MaxRing()
    { return 8; }
    body_type type()
    { return BODY_TYPE_WRAITH; }
    long size()
    { return 4000; }
    long weight()
    { return 2000; }
    OBJECT* weapon()
    { return &TheFist; }
    int damage()
    { return 10; }
    BODY_WRAITH(BINARY* binary) : (binary)  { }
    BODY_WRAITH() : (6)
    { AddAbility(new COUNTER_DEATH_PROTECTION); }
};

// @@ BODY_COCKATRICE

class BODY_COCKATRICE : public BODY
{
protected:
    int BodySpeed()
    { return 110; }
    const char* BodyName()
    { return "cockatrice"; }
    char LivingLook()
    { return 'c'; }
    int view()
    { return 3; }
    int constitution()
    { return 7; }
    int FoodValue()
    { return 6 * 60; }
    int stomach()
    { return 20 * 60; }
    void effect(LEVEL*, MONSTER* monster)
    { monster->AddAbility(new COUNTER_STONED(1)); }
    int armor()
    { return 20; }
public:
    object_type LookType()
    { return MONSTER_COCKATRICE; }
    void InsertAbility(MONSTER* monster);
    body_type type()
    { return BODY_TYPE_COCKATRICE; }
    long size()
    { return 750; }
    long weight()
    { return 500; }
    OBJECT* weapon()
    { return &TheCock; }
    int damage()
    { return 5; }
    cont hurt(int power, int type, OBJECT* weapon,
	      MONSTER* attack, MONSTER* defence, LEVEL* level);
    BODY_COCKATRICE(BINARY* binary) : (binary)  { }
    BODY_COCKATRICE() : (4)
    {
	AddAbility(new COUNTER_STONE_PROTECTION);
	AddAbility(new COUNTER_HISS);
    }
};

// @@ BODY_LIZARD

class BODY_LIZARD : public BODY
{
protected:
    int BodySpeed()
    { return 150; }
    const char* BodyName()
    { return "lizard"; }
    char LivingLook()
    { return 'l'; }
    int view()
    { return 3; }
    int constitution()
    { return 18; }
    int FoodValue()
    { return 4 * 60; }
    int stomach()
    { return 50 * 60; }
    bool ColdBlood()
    { return TRUE; }
    int armor()
    { return 50; }
public:
    object_type LookType()
    { return MONSTER_LIZARD; }
    body_type type()
    { return BODY_TYPE_LIZARD; }
    long size()
    { return 250; }
    long weight()
    { return 500; }
    OBJECT* weapon()
    { return &TheTeeth; }
    int damage()
    { return 7; }
    BODY_LIZARD(BINARY* binary) : (binary)  { }
    BODY_LIZARD() : (5)
    {
	AddAbility(new COUNTER_FIRE_PROTECTION);
	AddAbility(new COUNTER_POISON_PROTECTION);
    }
};

// @@ BODY_DRAGON

class BODY_DRAGON : public BODY
{
protected:
    int BodySpeed()
    { return 400; }
    const char* BodyName()
    { return "dragon"; }
    char LivingLook()
    { return 'D'; }
    int view()
    { return 4; }
    int constitution()
    { return 22; }
    int FoodValue()
    { return 750 * 60; }
    int stomach()
    { return 750 * 60; }
    bool poisonous()
    { return TRUE; }
    void effect(LEVEL*, MONSTER*)
    { AddAbility(new COUNTER_FIRE_PROTECTION); }
    bool ColdBlood()
    { return TRUE; }
    int armor()
    { return 90; }
public:
    object_type LookType()
    { return MONSTER_DRAGON; }
    int MaxRing()
    { return 4; }
    body_type type()
    { return BODY_TYPE_DRAGON; }
    long size()
    { return 75000; }
    long weight()
    { return 25000; }
    OBJECT* weapon()
    { return &TheTail; }
    int damage()
    { return 75; }
    BODY_DRAGON(BINARY* binary) : (binary)  { }
    BODY_DRAGON() : (22)
    {
	AddAbility(new COUNTER_FIRE_PROTECTION);
	AddAbility(new COUNTER_BREATHE_FIRE);
    }
};

// @@ BODY_ORC

class BODY_ORC : public BODY
{
protected:
    int BodySpeed()
    { return 90; }
    const char* BodyName()
    { return "orc"; }
    char LivingLook()
    { return 'O'; }
    int view()
    { return 3; }
    int constitution()
    { return 9; }
    int FoodValue()
    { return 30 * 60; }
    int stomach()
    { return 80 * 60; }
    int armor()
    { return 10; }
public:
    object_type LookType()
    { return MONSTER_ORC; }
    int MaxRing()
    { return 1; }
    body_type type()
    { return BODY_TYPE_ORC; }
    long size()
    { return 6000; }
    long weight()
    { return 4000; }
    OBJECT* weapon()
    { return &TheFist; }
    int damage()
    { return 10; }
    BODY_ORC(BINARY* binary) : (binary)  { }
    BODY_ORC() : (9) { }
};

// @@ BODY_HOBGOBLIN

class BODY_HOBGOBLIN : public BODY
{
protected:
    const char* BodyName()
    { return "hobgoblin"; }
    char LivingLook()
    { return 'H'; }
    int view()
    { return 4; }
    int constitution()
    { return 8; }
    int FoodValue()
    { return 20 * 60; }
    int stomach()
    { return 120 * 60; }
    int armor()
    { return 20; }
public:
    object_type LookType()
    { return MONSTER_HOBGOBLIN; }
    int MaxRing()
    { return 2; }
    body_type type()
    { return BODY_TYPE_HOBGOBLIN; }
    long size()
    { return 2000; }
    long weight()
    { return 2500; }
    OBJECT* weapon()
    { return &TheFist; }
    int damage()
    { return 8; }
    BODY_HOBGOBLIN(BINARY* binary) : (binary)  { }
    BODY_HOBGOBLIN() : (5) { }
};

// @@ BODY_GOBLIN

class BODY_GOBLIN : public BODY
{
protected:
    int BodySpeed()
    { return 110; }
    const char* BodyName()
    { return "goblin"; }
    char LivingLook()
    { return 'g'; }
    int view()
    { return 3; }
    int constitution()
    { return 6; }
    int FoodValue()
    { return 10 * 60; }
    int stomach()
    { return 60 * 60; }
    int armor()
    { return 10; }
public:
    object_type LookType()
    { return MONSTER_GOBLIN; }
    int MaxRing()
    { return 1; }
    body_type type()
    { return BODY_TYPE_GOBLIN; }
    long size()
    { return 1000; }
    long weight()
    { return 1500; }
    OBJECT* weapon()
    { return &TheFist; }
    int damage()
    { return 5; }
    BODY_GOBLIN(BINARY* binary) : (binary)  { }
    BODY_GOBLIN() : (4) { }
};

// @@ BODY_RAT

class BODY_RAT : public BODY
{
protected:
    int BodySpeed()
    { return 150; }
    const char* BodyName()
    { return "rat"; }
    char LivingLook()
    { return 'r'; }
    int view()
    { return 3; }
    int constitution()
    { return 2; }
    int FoodValue()
    { return 5 * 60; }
    int stomach()
    { return 25 * 60; }
    bool CanChoke()
    { return FALSE; }
    int armor()
    { return 8; }
public:
    bool hungry()
    { return TRUE; }
    object_type LookType()
    { return MONSTER_RAT; }
    body_type type()
    { return BODY_TYPE_RAT; }
    long size()
    { return 250; }
    long weight()
    { return 300; }
    OBJECT* weapon()
    { return &TheTeeth; }
    int damage()
    { return 1; }
    BODY_RAT(BINARY* binary) : (binary)  { }
    BODY_RAT() : (1)
    { AddAbility(new COUNTER_POISON_PROTECTION); }
};

// @@ BODY_BAT

class BODY_BAT : public BODY
{
protected:
    int BodySpeed()
    { return 250; }
    const char* BodyName()
    { return "bat"; }
    char LivingLook()
    { return 'b'; }
    int view()
    { return 5; }
    int constitution()
    { return 3; }
    int FoodValue()
    { return 2 * 60; }
    int stomach()
    { return 5 * 60; }
    void effect(LEVEL*, MONSTER* monster)
    { monster->AddAbility(new COUNTER_CONFUSED(rnd(50, 500))); }
    int armor()
    { return 5; }
public:
    object_type LookType()
    { return MONSTER_BAT; }
    body_type type()
    { return BODY_TYPE_BAT; }
    long size()
    { return 300; }
    long weight()
    { return 150; }
    OBJECT* weapon()
    { return &TheTeeth; }
    int damage()
    { return 2; }
    BODY_BAT(BINARY* binary) : (binary)  { }
    BODY_BAT() : (2)
    { AddAbility(new COUNTER_CONFUSED(0L, 20)); }
};

// @@ BODY_ACID_BLOB

class BODY_ACID_BLOB : public BODY
{
protected:
    int BodySpeed()
    { return 50; }
    const char* BodyName()
    { return "acid blob"; }
    char LivingLook()
    { return 'a'; }
    int view()
    { return 2; }
    int constitution()
    { return 7; }
    int FoodValue()
    { return 1 * 60; }
    int stomach()
    { return 10 * 60; }
    bool poisonous()
    { return TRUE; }
    int armor()
    { return 75; }
public:
    object_type LookType()
    { return MONSTER_ACID_BLOB; }
    bool CanFight(MONSTER* monster)
    { return monster->IsConfused(); }
    body_type type()
    { return BODY_TYPE_ACID_BLOB; }
    long size()
    { return 500; }
    long weight()
    { return 250; }
    OBJECT* weapon()
    { return &TheAcid; }
    int damage()
    { return 1; }
    cont hurt(int power, int type, OBJECT* weapon,
	      MONSTER* attack, MONSTER* defence, LEVEL* level);
    BODY_ACID_BLOB(BINARY* binary) : (binary)  { }
    BODY_ACID_BLOB() : (3)
    {
	AddAbility(new COUNTER_ACID_PROTECTION);
    }
};

// @@ end

#endif CORPSE_HEADER

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
