// @ myth - server - ray.C
//
// Myth is a multiuser screen oriented adventure game.
// Copyright (C) 1989 Per Abrahamsen
// 
// This file is part of Myth.
// 
// Myth is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 1, or (at your option) any later
// version.
// 
// Myth is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.
// 
// You should have received a copy of the GNU General Public License
// along with Myth; see the file COPYING.  If not, write to the Free
// Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.
//
// My email address is: amanda@iesd.auc.dk

// @@ include

#include "common/error.h"
#include "server/ray.h"
#include "server/level.h"
#include "server/monster.h"
#include "server/rnd.h"
#include "common/format.h"

// @@ Ray Weapons

COLD_BLOW TheColdBlow;
MAGIC_MISSILE TheMagicMissile;
FLAME TheFlame;
LIGHTENING_BOLT TheLighteningBolt;

// @@ class RAY

// @@@ void RAY::effect(LEVEL*, MONSTER*, coor)

void RAY::effect(LEVEL*, MONSTER*, coor)
{
    bug.error("RAY::effect", "virtual");
}

// @@@ cont RAY::zap(LEVEL* level, MONSTER* monster, coor pos)

cont RAY::zap(LEVEL* level, MONSTER* monster, coor pos)
{
    level->WatchEffect(pos, '*');
    effect(level, monster, pos);
    return (power-- < 1) ? DONE : CONTINUE;
}

// @@@ RAY::RAY(int value)

RAY::RAY(int value)
{ power = value * 2; }

// @@ class RAY_MONSTER

// @@@ void RAY_MONSTER::touch(LEVEL*, MONSTER*)

void RAY_MONSTER::touch(LEVEL*, MONSTER*)
{
    bug.error("RAY_MONSTER::touch", "virtual");
}

// @@@ void RAY_MONSTER::effect(LEVEL* level, MONSTER*, coor pos)

void RAY_MONSTER::effect(LEVEL* level, MONSTER*, coor pos)
{
    MONSTER_LIST list;
    level->GetMonster(&list, pos);
    
    list.reset();
    while(power > 0 && list.current() != NULL)
    {
	touch(level, list.current());
	power--;
	list.next();
    }
}

// @@ class RAY_FUNGUS

// @@@ void RAY_FUNGUS::touch(LEVEL* level, MONSTER* monster)

void RAY_FUNGUS::touch(LEVEL* level, MONSTER* monster)
{
    if(!monster->CheckAbility(ABILITY_POISON_PROTECTION) &&
       rnd.chance(power + MyLog(monster->size()), monster->experience() * 5))
    {
	monster->message("You are infected by the fungus.");
	level->WatchMonster(monster, "%s is infected by the fungus.");
	monster->AddAbility(new COUNTER_INFECTED(rnd(50, power * 100)));
    }
}

// @@ class RAY_SLEEP

// @@@ void RAY_SLEEP::touch(LEVEL* level, MONSTER* monster)

void RAY_SLEEP::touch(LEVEL* level, MONSTER* monster)
{
    monster->message(STRING("A sleep ray hits you!", STRING_STATIC));
    monster->message(STRING("", STRING_STATIC));
    monster->sleep(level, sqr(power) * 60);
}

// @@ class RAY_LIFE

// @@@ void RAY_LIFE::touch(LEVEL* level, MONSTER* monster)

void RAY_LIFE::touch(LEVEL* level, MONSTER* monster)
{
    OBJECT* object = monster->polymorph(level);
    
    if(object != NULL)
	bug.error("RAY_LIFE::touch", "Monster polymorph succeded!");
}

// @@ class RAY_DEATH

// @@@ void RAY_DEATH::touch(LEVEL* level, MONSTER* monster)

void RAY_DEATH::touch(LEVEL* level, MONSTER* monster)
{
    if(monster->CheckAbility(ABILITY_DEATH_PROTECTION))
	monster->message("You are uninjuried by the death ray!");
    else if(monster->CheckAbility(ABILITY_YENDOR))
	monster->message("The magic field of yendor protect you.");
    else
	monster->kill(STRING("was killed by a death ray", STRING_STATIC),
		      level);
}

// @@ class RAY_ABILITY

// @@@ COUNTER* RAY_ABILITY::ability()

COUNTER* RAY_ABILITY::ability()
{
    bug.error("RAY_ABILITY::ability", "virtual");
    return NULL;
}

// @@@ void RAY_ABILITY::touch(LEVEL*, MONSTER* monster)

void RAY_ABILITY::touch(LEVEL*, MONSTER* monster)
{
    monster->message(STRING("Some magic ray hits you", STRING_STATIC));
    monster->AddAbility(ability());
}

// @@ class RAY_SLOW

// @@@ COUNTER* RAY_SLOW::ability()

COUNTER* RAY_SLOW::ability()
{ return new COUNTER_SLOW(60 * 60 * power); }

// @@ class RAY_LIT

// @@@ COUNTER* RAY_LIT::ability()

COUNTER* RAY_LIT::ability()
{ return new COUNTER_LIGHT(60 * 60 * power); }

// @@ class RAY_FAST

// @@@ COUNTER* RAY_FAST::ability()

COUNTER* RAY_FAST::ability()
{ return new COUNTER_FAST(60 * 60 * power); }

// @@ class RAY_MISSILE

// @@@ OBJECT* RAY_MISSILE::weapon()

OBJECT* RAY_MISSILE::weapon()
{
    bug.error("RAY_MISSILE::weapon", "virtual");
    return NULL;
}

// @@@ void RAY_MISSILE::effect(LEVEL* level, MONSTER* monster, coor pos)

void RAY_MISSILE::effect(LEVEL* level, MONSTER* monster, coor pos)
{
    (void) level->hit(pos, power * 2, weapon(), monster, NoWhere);
}

// @@ class RAY_OBJECT

// @@@ cont RAY_OBJECT::touch(LEVEL*, coor, MONSTER*, BUNCH*, OBJECT*)

cont RAY_OBJECT::touch(LEVEL*, coor, MONSTER*, BUNCH*, OBJECT*)
{
    bug.error("RAY_OBJECT::touch", "virtual");
    return CONTINUE;
}

// @@@ void RAY_OBJECT::TouchMonster(LEVEL* level,

void RAY_OBJECT::TouchMonster(LEVEL* level, coor pos, MONSTER* monster,
			  BUNCH* bunch, MONSTER* subject)
{
    if(touch(level, pos, monster, bunch, subject) == DONE)
	bug.error("RAY_OBJECT::TouchMonster", "Destroy Monster");
}

// @@@ void RAY_OBJECT::effect(LEVEL* level, MONSTER* monster, coor pos)

void RAY_OBJECT::effect(LEVEL* level, MONSTER* monster, coor pos)
{
    BUNCH* bunch = level->bunch(pos);
    
    if(bunch == NULL)
	return;
    
    SPECIFY_ALL specify;
    OBJECT_LIST list;
    bunch->remove(&specify, &list, NULL, power);
    
    list.reset();
    while(power > 0 && list.current() != NULL)
    {
	int count = (int) list.current()->count();
	
	// touch() is responsible for putting the object back in the bunch
	if(list.current()->IsMonster())
	{
	    MONSTER* subject = (MONSTER*) list.current();
	    TouchMonster(level, pos, monster, bunch, subject);
	    list.next();
	}
	else if(touch(level, pos, monster, bunch, list.current()) == DONE)
	    list.erase();
	else
	    list.next();
	
	power -= count;
    }
}

// @@ class RAY_POLYMORPH

// @@@ cont RAY_POLYMORPH::touch(LEVEL* level, coor,

cont RAY_POLYMORPH::touch(LEVEL* level, coor,
			  MONSTER*, BUNCH* bunch, OBJECT* object)
{
    OBJECT* poly = object->polymorph(level);
    
    if(poly == NULL)
    {
	bunch->add(object);
	return CONTINUE;
    }
    else
    {
	bunch->add(poly);
	return DONE;
    }
}

// @@ class RAY_APPORT

// @@@ cont RAY_APPORT::touch(LEVEL* level, coor,

cont RAY_APPORT::touch(LEVEL* level, coor,
		       MONSTER* monster, BUNCH*, OBJECT* object)
{
    coor pos = monster->position();
    
    if(pos == NoWhere)
	return DONE;
    else
    {
	BUNCH* bunch = level->create(pos);
	bunch->add(object);
	return CONTINUE;
    }
}

// @@@ void RAY_APPORT::TouchMonster(LEVEL* level, coor,

void RAY_APPORT::TouchMonster(LEVEL* level, coor,
		  MONSTER* monster, BUNCH*, MONSTER* subject)
{
    coor pos = monster->position();
    
    if(pos == NoWhere)
	subject->kill(format("was summoned from limbo by %s",
			   ::print(monster, PRINT_MONSTER)), level);
    else
    {
	subject->move(level, pos);
	subject->teleport(level, 2);
    }
}

// @@ class RAY_TELEPORT

// @@@ cont RAY_TELEPORT::touch(LEVEL* level, coor pos,

cont RAY_TELEPORT::touch(LEVEL* level, coor pos,
			 MONSTER*, BUNCH*, OBJECT* object)
{
    object->MoveAway(level, pos, power + 1);
    return CONTINUE;
}

// @@@ void RAY_TELEPORT::TouchMonster(LEVEL* level, coor,

void RAY_TELEPORT::TouchMonster(LEVEL* level, coor,
				MONSTER*, BUNCH*, MONSTER* subject)
{
    if(subject->CheckAbility(ABILITY_TELEPORT_CONTROL))
	subject->teleport(level, power, BLESSED);
    else
	subject->teleport(level, power, NORMAL);
}

// @@ class RAY_LEVEL_TELEPORT

// @@@ cont RAY_LEVEL_TELEPORT::touch(LEVEL*, coor, MONSTER*, BUNCH*, OBJECT*)

cont RAY_LEVEL_TELEPORT::touch(LEVEL*, coor, MONSTER*, BUNCH*, OBJECT*)
{
    return DONE;
}

// @@@ void RAY_LEVEL_TELEPORT::TouchMonster(LEVEL*, coor,

void RAY_LEVEL_TELEPORT::TouchMonster(LEVEL*, coor,
				      MONSTER*, BUNCH*, MONSTER* subject)
{
    subject->message(STRING("A magic ray send you through the dungeon",
			    STRING_STATIC));
    subject->NewLevel((level_type) rnd(1, global.DungeonSize() - 1), '<');
}

// @@ class RAY_INVISIBLE

// @@@ cont RAY_INVISIBLE::touch(LEVEL*, coor, MONSTER*,

cont RAY_INVISIBLE::touch(LEVEL*, coor, MONSTER*, BUNCH* bunch, OBJECT* object)
{
    object->ClearVisible();
    bunch->add(object);
    return CONTINUE;
}

// @@ class RAY_VISIBLE

// @@@ cont RAY_VISIBLE::touch(LEVEL* level, coor,

cont RAY_VISIBLE::touch(LEVEL*, coor, MONSTER*, BUNCH* bunch, OBJECT* object)
{
    object->SetVisible();
    bunch->add(object);
    return CONTINUE;
}

// @@ class RAY_LIGHT

// @@@ void RAY_LIGHT::effect(LEVEL* level, MONSTER*, coor pos)

void RAY_LIGHT::effect(LEVEL* level, MONSTER*, coor pos)
{
    level->light(pos);
}

// @@ class RAY_DARK

// @@@ void RAY_DARK::effect(LEVEL* level, MONSTER*, coor pos)

void RAY_DARK::effect(LEVEL* level, MONSTER*, coor pos)
{
    level->dark(pos);
}

// @@ emacs
//
// Local Variables:
// mode: c++
// outline-regexp: "// @+ "
// eval: (outline-minor-mode 1)
// End:
